# Generated by Django 3.2.19 on 2023-06-07 11:07

from django.db import migrations, models
import django_extensions.db.fields
import opaque_keys.edx.django.models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='CourseDetails',
            fields=[
                ('created', django_extensions.db.fields.CreationDateTimeField(auto_now_add=True, verbose_name='created')),
                ('modified', django_extensions.db.fields.ModificationDateTimeField(auto_now=True, verbose_name='modified')),
                ('id', opaque_keys.edx.django.models.CourseKeyField(db_index=True, help_text='Courserun key', max_length=255, primary_key=True, serialize=False)),
                ('course_type', models.CharField(help_text='Type of course. For example Masters, Verified, Audit, executive-education-2u,  etc', max_length=255)),
                ('product_source', models.CharField(help_text='Tells about the origin of a course. For example, edx, 2u', max_length=255)),
                ('start_date', models.DateTimeField(help_text='The start date of the course.', null=True)),
                ('end_date', models.DateTimeField(help_text='The end date of the course.', null=True)),
                ('enroll_by', models.DateTimeField(help_text='The suggested deadline for enrollment.', null=True)),
            ],
        ),
    ]
