# Generated by Django 4.2.14 on 2024-07-25 18:06

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('federated_content_connector', '0003_reset_refresh_timestamp'),
    ]

    operations = [
        migrations.AddField(
            model_name='coursedetails',
            name='course_key',
            field=models.CharField(blank=True, default='', help_text='The top top level course key associated with the courserun key', max_length=255),
        ),
    ]
