"""Constants for the federated content connector."""

EXEC_ED_LANDING_PAGE = 'https://www.getsmarter.com/account'
EXEC_ED_COURSE_TYPE = 'executive-education-2u'
BOOTCAMP_2U = 'bootcamp-2u'
PRODUCT_SOURCE_2U = '2u'
